@@ -26,7 +26,7 @@
 
 struct dfa_comp
 {
-  /* KWset compiled pattern.  For Ecompile and Gcompile, we compile
+  /* KWset compiled pattern.  For GEAcompile, we compile
      a list of strings, at least one of which is known to occur in
      any string matching the regexp. */
   kwset_t kwset;
@@ -145,7 +145,8 @@ possible_backrefs_in_pattern (char const *keys, ptrdiff_t len, bool bs_safe)
 
 static bool
 regex_compile (struct dfa_comp *dc, char const *p, ptrdiff_t len,
-               ptrdiff_t pcount, ptrdiff_t lineno, bool syntax_only)
+               ptrdiff_t pcount, ptrdiff_t lineno, reg_syntax_t syntax_bits,
+               bool syntax_only)
 {
   struct re_pattern_buffer pat0;
   struct re_pattern_buffer *pat = syntax_only ? &pat0 : &dc->patterns[pcount];
@@ -157,14 +158,19 @@ regex_compile (struct dfa_comp *dc, char const *p, ptrdiff_t len,
 
   pat->translate = NULL;
 
+  if (syntax_only)
+    re_set_syntax (syntax_bits | RE_NO_SUB);
+  else
+    re_set_syntax (syntax_bits);
+
   char const *err = re_compile_pattern (p, len, pat);
   if (!err)
     return true;
 
   /* Emit a filename:lineno: prefix for patterns taken from files.  */
-  size_t pat_lineno = lineno;
+  size_t pat_lineno;
   char const *pat_filename
-    = lineno < 0 ? "\0" : pattern_file_name (lineno + 1, &pat_lineno);
+    = lineno < 0 ? "" : pattern_file_name (lineno, &pat_lineno);
 
   if (*pat_filename == '\0')
     error (0, 0, "%s", err);
@@ -174,8 +180,13 @@ regex_compile (struct dfa_comp *dc, char const *p, ptrdiff_t len,
   return false;
 }
 
+/* Compile PATTERN, containing SIZE bytes that are followed by '\n'.
+   SYNTAX_BITS specifies whether PATTERN uses style -G, -E, or -A.
+   Return a description of the compiled pattern.  */
+
 void *
-GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
+GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits,
+            bool exact)
 {
   char *motif;
   struct dfa_comp *dc = xcalloc (1, sizeof (*dc));
@@ -184,7 +195,6 @@ GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
 
   if (match_icase)
     syntax_bits |= RE_ICASE;
-  re_set_syntax (syntax_bits);
   int dfaopts = eolbyte ? 0 : DFA_EOL_NUL;
   dfasyntax (dc->dfa, &localeinfo, syntax_bits, dfaopts);
   bool bs_safe = !localeinfo.multibyte | localeinfo.using_utf8;
@@ -213,15 +223,8 @@ GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
 
   do
     {
-      size_t len;
-      char const *sep = memchr (p, '\n', patlim - p);
-      if (sep)
-        {
-          len = sep - p;
-          sep++;
-        }
-      else
-        len = patlim - p;
+      char const *sep = rawmemchr (p, '\n');
+      ptrdiff_t len = sep - p;
 
       bool backref = possible_backrefs_in_pattern (p, len, bs_safe);
 
@@ -244,10 +247,13 @@ GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
           dc->patterns++;
         }
 
-      if (!regex_compile (dc, p, len, dc->pcount, lineno, !backref))
+      re_set_syntax (syntax_bits);
+
+      if (!regex_compile (dc, p, len, dc->pcount, lineno, syntax_bits,
+                          !backref))
         compilation_failed = true;
 
-      p = sep;
+      p = sep + 1;
       lineno++;
 
       if (backref)
@@ -256,12 +262,12 @@ GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
           prev = p;
         }
     }
-  while (p);
+  while (p <= patlim);
 
   if (compilation_failed)
     exit (EXIT_TROUBLE);
 
-  if (prev != NULL)
+  if (prev <= patlim)
     {
       if (pattern < prev)
         {
@@ -277,18 +283,6 @@ GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
         }
     }
 
-  if (buf != NULL)
-    {
-      dc->patterns--;
-      dc->pcount++;
-
-      if (!regex_compile (dc, buf, buflen, 0, -1, false))
-        abort ();
-
-      if (buf != pattern)
-        free (buf);
-    }
-
   /* In the match_words and match_lines cases, we use a different pattern
      for the DFA matcher that will quickly throw out cases that won't work.
      Then if DFA succeeds we do some hairy stuff using the regex matcher
@@ -324,6 +318,21 @@ GEAcompile (char *pattern, size_t size, reg_syntax_t syntax_bits)
   kwsmusts (dc);
   dfacomp (NULL, 0, dc->dfa, 1);
 
+  if (buf != NULL)
+    {
+      if (exact || !dfasupported (dc->dfa))
+        {
+          dc->patterns--;
+          dc->pcount++;
+
+          if (!regex_compile (dc, buf, buflen, 0, -1, syntax_bits, false))
+            abort ();
+        }
+
+      if (buf != pattern)
+        free (buf);
+    }
+
   free (motif);
 
   return dc;
@@ -383,14 +392,19 @@ EGexecute (void *vdc, char const *buf, size_t size, size_t *match_size,
                  greater of the latter two values; this temporarily prefers
                  the DFA to KWset.  */
               exact_kwset_match = kwsm.index < dc->kwset_exact_matches;
-              end = ((exact_kwset_match || !dfafast
-                      || MAX (16, match - beg) < (match - prev_beg) >> 2)
-                     ? match
-                     : MAX (16, match - beg) < (buflim - prev_beg) >> 2
-                     ? prev_beg + 4 * MAX (16, match - beg)
-                     : buflim);
-              end = memchr (end, eol, buflim - end);
-              end = end ? end + 1 : buflim;
+              if (exact_kwset_match || !dfafast
+                  || MAX (16, match - beg) < (match - prev_beg) >> 2)
+                {
+                  end = rawmemchr (match, eol);
+                  end++;
+                }
+              else if (MAX (16, match - beg) < (buflim - prev_beg) >> 2)
+                {
+                  end = rawmemchr (prev_beg + 4 * MAX (16, match - beg), eol);
+                  end++;
+                }
+              else
+                end = buflim;
 
               if (exact_kwset_match)
                 {
@@ -425,8 +439,8 @@ EGexecute (void *vdc, char const *buf, size_t size, size_t *match_size,
                   beg++;
                   dfa_beg = beg;
                 }
-              end = memchr (next_beg, eol, buflim - next_beg);
-              end = end ? end + 1 : buflim;
+              end = rawmemchr (next_beg, eol);
+              end++;
 
               count = 0;
             }
@@ -446,8 +460,8 @@ EGexecute (void *vdc, char const *buf, size_t size, size_t *match_size,
               beg = memrchr (buf, eol, next_beg - buf);
               beg++;
             }
-          end = memchr (next_beg, eol, buflim - next_beg);
-          end = end ? end + 1 : buflim;
+          end = rawmemchr (next_beg, eol);
+          end++;
 
           /* Successful, no back-references encountered! */
           if (!backref)
